package cells;
public class Organism extends Thread
{
    private Cell[][] cells;
    private CellularAutomatCanvas canvas;

    public Organism(Cell[][] cells, CellularAutomatCanvas canvas)
    {
        this.cells = cells;
        this.canvas = canvas;

        // we use the organism thread to calculate the neighbours for each cell
        start();
    }

    // calculates the neighbours for every cell in organism
    public void setNeighbours()
    {
        int x;
        int n = cells.length;
        // edge cells
        // cell [0][0]
        x = 0;
        if (cells[0][1].getLife()) x++;
        if (cells[1][1].getLife()) x++;
        if (cells[1][0].getLife()) x++;
        cells[0][0].setNumberOfNeighbours(x);

        // cell [0][n-1]
        x = 0;
        if (cells[0][n-2].getLife()) x++;
        if (cells[1][n-2].getLife()) x++;
        if (cells[1][n-1].getLife()) x++;
        cells[0][n-1].setNumberOfNeighbours(x);

        // cell [n-1][0]
        x = 0;
        if (cells[n-2][0].getLife()) x++;
        if (cells[n-2][1].getLife()) x++;
        if (cells[n-1][1].getLife()) x++;
        cells[n-1][0].setNumberOfNeighbours(x);

        // cell [n-1][n-1]
        x = 0;
        if (cells[n-2][n-1].getLife()) x++;
        if (cells[n-2][n-2].getLife()) x++;
        if (cells[n-1][n-2].getLife()) x++;
        cells[n-1][0].setNumberOfNeighbours(x);


        // border cells
        for (int j = 1; j < cells.length - 1; j++)
        {
            // row 0
            x = 0;
            if (cells[0][j - 1].getLife()) x++;
            if (cells[0][j].getLife()) x++;
            if (cells[0][j + 1].getLife()) x++;
            if (cells[1][j - 1].getLife()) x++;
            if (cells[1][j].getLife()) x++;
            if (cells[1][j + 1].getLife()) x++;
            cells[0][j].setNumberOfNeighbours(x);

            // row n-1
            x = 0;
            if (cells[n - 1][j - 1].getLife()) x++;
            if (cells[n - 1][j].getLife()) x++;
            if (cells[n - 1][j + 1].getLife()) x++;
            if (cells[n - 1][j - 1].getLife()) x++;
            if (cells[n - 1][j].getLife()) x++;
            if (cells[n - 1][j + 1].getLife()) x++;
            cells[n - 1][j].setNumberOfNeighbours(x);

            // collumn 0 -> [j][0]
            x = 0;
            if (cells[j - 1][0].getLife()) x++;
            if (cells[j - 1][1].getLife()) x++;
            if (cells[j][1].getLife()) x++;
            if (cells[j + 1][0].getLife()) x++;
            if (cells[j + 1][1].getLife()) x++;
            cells[j][0].setNumberOfNeighbours(x);

            // collumn n-1 -> [j][n-1]
            x = 0;
            if (cells[j - 1][n - 1].getLife()) x++;
            if (cells[j - 1][n - 2].getLife()) x++;
            if (cells[j][n - 2].getLife()) x++;
            if (cells[j + 1][n - 2].getLife()) x++;
            if (cells[j + 1][n - 1].getLife()) x++;
            cells[j][n - 1].setNumberOfNeighbours(x);
        }

        // inside cells
        for (int i = 1; i < cells.length - 1; i++)
        {
            for(int j = 1; j < cells.length - 1; j++)
            {
                x = 0;
                if (cells[i-1][j - 1].getLife()) x++;
                if (cells[i-1][j].getLife()) x++;
                if (cells[i-1][j+1].getLife()) x++;
                if (cells[i][j - 1].getLife()) x++;
                if (cells[i][j + 1].getLife()) x++;
                if (cells[i + 1][j - 1].getLife()) x++;
                if (cells[i + 1][j].getLife()) x++;
                if (cells[i + 1][j + 1].getLife()) x++;
                cells[i][j].setNumberOfNeighbours(x);
            }
        }
    }

    @Override
    public void run()
    {
        canvas.repaint();

        // we will wait first for everything to initialize
        try {
            Thread.sleep(1000);
        } catch (InterruptedException e) {
            e.printStackTrace();
        }

        // and then we have some fun!
        while (true)
        {
            try {
                Thread.sleep(500);
                setNeighbours();
                canvas.repaint();
            } catch (InterruptedException e) {
                e.printStackTrace();
            }
        }
    }
}
